@@ -32,6 +32,9 @@
 #include "usb.h"
 #include "usb_standard_request.h"
 
+#include <rom_iap.h>
+#include "usb_descriptor.h"
+
 #include "usb_device.h"
 #include "usb_endpoint.h"
 #include "usb_api_board_info.h"
@@ -96,6 +99,11 @@ usb_request_status_t usb_vendor_request_set_transceiver_mode(
 			set_transceiver_mode(endpoint->setup.value);
 			usb_transfer_schedule_ack(endpoint->in);
 			return USB_REQUEST_STATUS_OK;
+		case TRANSCEIVER_MODE_CPLD_UPDATE:
+			usb_endpoint_init(&usb_endpoint_bulk_out);
+			start_cpld_update = true;
+			usb_transfer_schedule_ack(endpoint->in);
+			return USB_REQUEST_STATUS_OK;
 		default:
 			return USB_REQUEST_STATUS_STALL;
 		}
@@ -183,6 +191,31 @@ void usb_configuration_changed(
 	}
 }
 
+void usb_set_descriptor_by_serial_number(void)
+{
+	iap_cmd_res_t iap_cmd_res;
+	
+	/* Read IAP Serial Number Identification */
+	iap_cmd_res.cmd_param.command_code = IAP_CMD_READ_SERIAL_NO;
+	iap_cmd_call(&iap_cmd_res);
+	
+	if (iap_cmd_res.status_res.status_ret == CMD_SUCCESS) {
+		usb_descriptor_string_serial_number[0] = USB_DESCRIPTOR_STRING_SERIAL_BUF_LEN;
+		usb_descriptor_string_serial_number[1] = USB_DESCRIPTOR_TYPE_STRING;
+		
+		/* 32 characters of serial number, convert to UTF-16LE */
+		for (size_t i=0; i<USB_DESCRIPTOR_STRING_SERIAL_LEN; i++) {
+			const uint_fast8_t nibble = (iap_cmd_res.status_res.iap_result[i >> 3] >> (28 - (i & 7) * 4)) & 0xf;
+			const char c = (nibble > 9) ? ('a' + nibble - 10) : ('0' + nibble);
+			usb_descriptor_string_serial_number[2 + i * 2] = c;
+			usb_descriptor_string_serial_number[3 + i * 2] = 0x00;
+		}
+	} else {
+		usb_descriptor_string_serial_number[0] = 2;
+		usb_descriptor_string_serial_number[1] = USB_DESCRIPTOR_TYPE_STRING;
+	}
+}
+
 int main(void) {
 	pin_setup();
 	enable_1v8_power();
@@ -191,6 +224,8 @@ int main(void) {
 #endif
 	cpu_clock_init();
 
+	usb_set_descriptor_by_serial_number();
+
 	usb_set_configuration_changed_cb(usb_configuration_changed);
 	usb_peripheral_reset();
 	
